/****************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 9 $
 * $Date: 14/10/06 9:56a $
 * @brief    Transmit and receive data from PC terminal through RS232 interface.
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "includes.h"
#include "NUC472_442.h"

#include "mb.h"
#include "mb_m.h"

#define PLLCON_SETTING      CLK_PLLCTL_84MHz_HXT
#define PLL_CLOCK           84000000

//START 
//ȼ
#define START_TASK_PRIO			10  ///ʼȼΪ
//ջС
#define START_STK_SIZE			128
//ջ
OS_STK START_TASK_STK[START_STK_SIZE];
//
void start_task(void *pdata);

//MASTER
//ȼ
#define MASTER_TASK_PRIO			4
//ջС
#define MASTER_STK_SIZE				64
//ջ
OS_STK MASTER_TASK_STK[MASTER_STK_SIZE];
//
void MASTER_task(void *pdata);

//LED1
//ȼ
#define SLAVE_TASK_PRIO			6
//ջС
#define SLAVE_STK_SIZE				64
//ջ
OS_STK SLAVE_TASK_STK[SLAVE_STK_SIZE];
//
void SLAVE_task(void *pdata);

//
#define DataChange_TASK_PRIO			5
//ջС
#define DataChange_STK_SIZE			128
//ջ
OS_STK DataChange_TASK_STK[DataChange_STK_SIZE]; 
//
void DataChange_task(void *pdata);

/*---------------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void);

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable External XTAL (4~24 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady( CLK_STATUS_HXTSTB_Msk);

    /* Switch HCLK clock source to HXT */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HXT,CLK_CLKDIV0_HCLK(1));

    /* Set PLL to power down mode and PLL_STB bit in CLKSTATUS register will be cleared by hardware.*/
    CLK->PLLCTL|= CLK_PLLCTL_PD_Msk;

    /* Set PLL frequency */
    CLK->PLLCTL = PLLCON_SETTING;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_PLLSTB_Msk);

    /* Switch HCLK clock source to PLL */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL,CLK_CLKDIV0_HCLK(1));

    /* Enable IP clock */
    CLK_EnableModuleClock(UART0_MODULE);
    CLK_EnableModuleClock(UART1_MODULE);
    CLK_EnableModuleClock(TMR0_MODULE);
    CLK_EnableModuleClock(TMR1_MODULE);    

    /* Select IP clock source */
    CLK_SetModuleClock(UART0_MODULE,CLK_CLKSEL1_UARTSEL_HXT,CLK_CLKDIV0_UART(1));
    CLK_SetModuleClock(UART1_MODULE,CLK_CLKSEL1_UARTSEL_HXT,CLK_CLKDIV0_UART(1));
    CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0SEL_HXT, 0);
    CLK_SetModuleClock(TMR1_MODULE, CLK_CLKSEL1_TMR1SEL_HXT, 0);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PG multi-function pins for UART0 RXD, TXD */
    SYS->GPG_MFPL &= ~(SYS_GPG_MFPL_PG1MFP_Msk | SYS_GPG_MFPL_PG2MFP_Msk);
    SYS->GPG_MFPL |= (SYS_GPG_MFPL_PG1MFP_UART0_RXD | SYS_GPG_MFPL_PG2MFP_UART0_TXD);

    /* Set PB multi-function pins for UART1 RXD, TXD*/
    SYS->GPB_MFPL &= ~(SYS_GPB_MFPL_PB2MFP_Msk | SYS_GPB_MFPL_PB3MFP_Msk);
    SYS->GPB_MFPL |= (SYS_GPB_MFPL_PB2MFP_UART1_RXD | SYS_GPB_MFPL_PB3MFP_UART1_TXD);

    /* Lock protected registers */
    SYS_LockReg();

}

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int main(void)
{
    SYS_Init();
  	OSInit();  //UCOSʼ
	OSTaskCreate(start_task,(void*)0,(OS_STK*)&START_TASK_STK[START_STK_SIZE-1],START_TASK_PRIO); //ʼ
	OSStart(); //ʼ
}

//ʼ
void start_task(void *pdata)
{
	OS_CPU_SR cpu_sr=0;
	pdata=pdata;
    SysTick_Config(84000);
	
	OSStatInit();  //ͳ
	
	OS_ENTER_CRITICAL();  //ٽ(رж)
	OSTaskCreate(MASTER_task,(void*)0,(OS_STK*)&MASTER_TASK_STK[MASTER_STK_SIZE-1],MASTER_TASK_PRIO);               //ѵ
	OSTaskCreate(SLAVE_task,(void*)0,(OS_STK*)&SLAVE_TASK_STK[SLAVE_STK_SIZE-1],SLAVE_TASK_PRIO);                   //ӻѵ
	OSTaskCreate(DataChange_task,(void*)0,(OS_STK*)&DataChange_TASK_STK[DataChange_STK_SIZE-1],DataChange_TASK_PRIO);//ݸıȡ
	OSTaskSuspend(START_TASK_PRIO);//ʼ
	OS_EXIT_CRITICAL();  //˳ٽ(ж)
}

//ӻѵ
void SLAVE_task(void *pdata)
{
	pdata=pdata;
    eMBInit( MB_RTU, 0x01, 0, 115200, MB_PAR_NONE );
    eMBEnable( );   
	while(1)
	{
        eMBPoll();
		OSTimeDlyHMSM (0, 0, 0, 50);
	};
}

//ѵ
void MASTER_task(void *pdata)
{
	pdata=pdata;
    eMBMasterInit( MB_RTU, 0 , 115200, MB_PAR_NONE);
    eMBMasterEnable();
	while(1)
	{
        eMBMasterPoll();
        OSTimeDlyHMSM (0, 0, 0, 10);
	}
}

//ݸı
void DataChange_task(void *pdata)
{
    uint8_t  Time;
    uint8_t  Coli[2];
    uint16_t usModbusUserData[6] ;
	OS_CPU_SR cpu_sr=0;
    
    Coli[0] = 0xA5;
    Coli[1] = 0;
    for(Time = 0;Time <6; Time ++)
      usModbusUserData[Time ] = Time *100;
	pdata=pdata;
    
    GPIO_SetMode(PB, BIT10, GPIO_MODE_OUTPUT);
	while(1)
	{
        PB10 = 0;
        usModbusUserData[0] ++;
        Coli[1] ++;
        eMBMasterReqWriteMultipleHoldingRegister(1,1,6,usModbusUserData);        
		OSTimeDlyHMSM (0, 0, 0, 100);
        PB10 = 1;
        eMBMasterReqWriteMultipleCoils(1, 0, 16,Coli);
        OSTimeDlyHMSM (0, 0, 0, 100);
	}
}

void SysTick_Handler(void)
{
    OSIntEnter();		//ж
    OSTimeTick();       //ucosʱӷ               
    OSIntExit();        //лж
}
