/*
 * FreeModbus Libary: STM32 Port
 * Copyright (C) 2013 Armink <armink.ztl@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: portserial_m.c,v 1.60 2013/08/13 15:07:05 Armink add Master Functions $
 */

#include "includes.h"
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mb_m.h"
#include "mbport.h"
#include "mbconfig.h"

#if MB_MASTER_RTU_ENABLED > 0 || MB_MASTER_ASCII_ENABLED
/* ----------------------- static functions ---------------------------------*/
static void prvvUARTTxReadyISR(void);
static void prvvUARTRxISR(void);
/* ----------------------- Start implementation -----------------------------*/

void vMBMasterPortSerialEnable(BOOL xRxEnable, BOOL xTxEnable)
{
	__disable_irq();
	
	if( xRxEnable )
    {
       UART_ENABLE_INT(UART1, UART_INTEN_RDAIEN_Msk); 
    }
    else
    {
  		  UART_DISABLE_INT(UART1, UART_INTEN_RDAIEN_Msk);
    }
    if( xTxEnable )
    {
         UART_ENABLE_INT(UART1, UART_INTEN_THREIEN_Msk);
    }
    else
    {
      UART_DISABLE_INT(UART1, UART_INTEN_THREIEN_Msk);
    }
	
	
	NVIC_EnableIRQ(UART1_IRQn);
	__enable_irq();
}

void vMBMasterPortClose(void)
{
    /* Disable Interrupt */
    UART_DISABLE_INT(UART1, UART_INTEN_RDAIEN_Msk | UART_INTEN_THREIEN_Msk);

	/* Disable I & F bit */ 
  	NVIC_DisableIRQ(UART1_IRQn);
}
//Ĭһ 2 ʿ  ż
BOOL xMBMasterPortSerialInit(UCHAR ucPORT, ULONG ulBaudRate, UCHAR ucDataBits,
		eMBParity eParity)
{
	if (ucPORT!=0) return FALSE;
    UART_Open(UART1, ulBaudRate);
    return TRUE; 
}

BOOL xMBMasterPortSerialPutByte(CHAR ucByte)
{
    UART_WRITE(UART1 , ucByte);				
    return TRUE;
}

BOOL xMBMasterPortSerialGetByte(CHAR * pucByte)
{
    *pucByte=UART_READ(UART1 );	    
    return TRUE;
}

/* 
 * Create an interrupt handler for the transmit buffer empty interrupt
 * (or an equivalent) for your target processor. This function should then
 * call pxMBFrameCBTransmitterEmpty( ) which tells the protocol stack that
 * a new character can be sent. The protocol stack will then call 
 * xMBPortSerialPutByte( ) to send the character.
 */
void prvvUARTTxReadyISR(void)
{
	pxMBMasterFrameCBTransmitterEmpty();
}

/* 
 * Create an interrupt handler for the receive interrupt for your target
 * processor. This function should then call pxMBFrameCBByteReceived( ). The
 * protocol stack will then call xMBPortSerialGetByte( ) to retrieve the
 * character.
 */
void prvvUARTRxISR(void)
{
	pxMBMasterFrameCBByteReceived();
}
/*******************************************************************************
 * Function Name  : USART2_IRQHandler
 * Description    : This function handles USART2 global interrupt request.
 * Input          : None
 * Output         : None
 * Return         : None
 *******************************************************************************/
void UART1_IRQHandler(void)
{
   uint32_t u32IntSts = UART1->INTSTS;
    OSIntEnter();		//ж 
  if(u32IntSts & UART_INTSTS_THREINT_Msk){
	          prvvUARTTxReadyISR();//жӳ 
	}
	else if(u32IntSts & UART_INTSTS_RDAINT_Msk){
				while(UART_IS_RX_READY(UART1))
	        	prvvUARTRxISR();//жӳ 
	}
     OSIntExit();        //лж
}

#endif
